# frozen_string_literal: true

require "opentelemetry"
require "opentelemetry-propagator-b3"

module Aspecto
  module OpenTelemetry
    module Propagator
      # Aspecto OpenTelemetry Propagator Configuration
      module Aspecto
        extend self

        W3C_PROPAGATOR = ::OpenTelemetry::Trace::Propagation::TraceContext.text_map_propagator
        B3_SINGLE_PROPAGATOR = ::OpenTelemetry::Propagator::B3::Single.text_map_propagator
        B3_MULTI_PROPAGATOR = ::OpenTelemetry::Propagator::B3::Multi.text_map_propagator

        def from_configurator(configurator)
          injectors = [W3C_PROPAGATOR]
          injectors.push(B3_SINGLE_PROPAGATOR) if configurator.inject_b3_context_single_header
          injectors.push(B3_MULTI_PROPAGATOR) if configurator.inject_b3_context_multi_header

          extractors = [W3C_PROPAGATOR]
          extractors.push(B3_SINGLE_PROPAGATOR, B3_MULTI_PROPAGATOR) if configurator.extract_b3_context

          ::OpenTelemetry::Context::Propagation::CompositeTextMapPropagator.compose(injectors: injectors, extractors: extractors)
        end
      end
    end
  end
end
