# frozen_string_literal: true

require "opentelemetry/sdk"
require_relative "./condition"

module Aspecto
  # Aspecto OpenTelemetry Distro
  module OpenTelemetry
    module Sampler
      # Single rule for sampling
      class SamplingRule
        def initialize(rule_config)
          @id = rule_config["_id"]
          @sampler = ::OpenTelemetry::SDK::Trace::Samplers.trace_id_ratio_based(rule_config["samplingRate"])
          @conditions = rule_config["conditions"].map { |condition_config| Condition.new condition_config }
          @context = rule_config["context"]
        end

        attr_reader :sampler, :context

        def satisfies?(attributes, span_name)
          @conditions.all? { |condition| condition.satisfies?(attributes, span_name) }
        end
      end
    end
  end
end
